//! Configuration data.

use std::path::{Path, PathBuf};

use thiserror::Error;

use crate::cli::Cli;

/// Errors when dealing with application configuration.
#[derive(Error, Debug)]
pub enum ConfigError {
    /// Calibre library path does not exist.
    #[error("no folder at {0}")]
    LibraryPathNotFound(String),
    /// Calibre database does not exist.
    #[error("no metadata.db in {0}")]
    MetadataNotFound(String),
}

/// Application configuration.
pub struct Config {
    /// Calibre library folder.
    pub library_path: PathBuf,
    /// Calibre metadata file path.
    pub metadata_path: PathBuf,
}

impl Config {
    /// Check if the calibre library from `args` exists and if the calibre database can be found.
    pub fn load(args: &Cli) -> Result<Self, ConfigError> {
        let library_path = Path::new(&args.library_path).to_path_buf();

        if !library_path.exists() {
            let library_path = library_path
                .as_os_str()
                .to_str()
                .unwrap_or("<failed to parse path>")
                .to_string();
            return Err(ConfigError::LibraryPathNotFound(library_path));
        }

        let metadata_path = library_path.join("metadata.db");
        if !metadata_path.exists() {
            let metadata_path = metadata_path
                .as_os_str()
                .to_str()
                .unwrap_or("<failed to parse path>")
                .to_string();
            return Err(ConfigError::MetadataNotFound(metadata_path));
        }

        Ok(Self {
            library_path,
            metadata_path,
        })
    }
}
