//! Handle requests for specific formats of a book.

use std::sync::Arc;

use tokio::fs::File;

use poem::{
    error::NotFoundError,
    handler,
    web::{Data, Path},
    Body, IntoResponse, Response,
};
use tokio_util::io::ReaderStream;

use crate::{
    app_state::AppState,
    data::book::{Book, Format},
    handlers::error::HandlerError,
    opds::media_type::MediaType,
};

/// Handle a request for a book with id `id` in format `format`.
#[handler]
pub async fn handler(
    Path((id, format)): Path<(u64, String)>,
    state: Data<&Arc<AppState>>,
) -> Result<Response, poem::Error> {
    let book = state
        .calibre
        .scalar_book(id)
        .map_err(HandlerError::DataError)?;
    let book = Book::full_book(&book, &state).ok_or(NotFoundError)?;
    let format = Format(format);
    let file_name = book.formats.get(&format).ok_or(NotFoundError)?;
    let file_path = state
        .config
        .library_path
        .join(book.data.path)
        .join(file_name);

    let mut file = File::open(file_path).await.map_err(|_| NotFoundError)?;
    let stream = ReaderStream::new(file);
    let body = Body::from_bytes_stream(stream);

    let content_type: MediaType = format.into();
    Ok(body
        .with_content_type(format!("{content_type}"))
        .with_header("Content-Disposition", format!("filename=\"{file_name}\""))
        .into_response())
}
